﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using System.ComponentModel;

namespace DeviceManagerMonitor
{
    internal class HardwareInfo
    {
        private string mHardType;

        [Browsable(false)]
        [DisplayName("硬件类型")]
        public string HardType
        {
            get { return mHardType; }
            set { mHardType = value; }
        }

        private string mHardName;

        [Browsable(false)]
        [DisplayName("硬件名")]
        public string HardName
        {
            get { return mHardName; }
            set { mHardName = value; }
        }

        private string mHardDescription;

        [Browsable(false)]
        [DisplayName("硬件描述")]
        public string HardDescription
        {
            get { return mHardDescription; }
            set { mHardDescription = value; }
        }

        private string mHardClassGuid;

        [Browsable(false)]
        [DisplayName("全球唯一标识符")]
        public string HardClassGuid
        {
            get { return mHardClassGuid; }
            set { mHardClassGuid = value; }
        }

        private string mHardManufacturer;

        [Browsable(false)]
        [DisplayName("制造商")]
        public string HardManufacturer
        {
            get { return mHardManufacturer; }
            set { mHardManufacturer = value; }
        }

        private string mHardDeviceID;

        [Browsable(false)]
        [DisplayName("设备实例路径，DeviceId(地址或其他标识信息，以便唯一地命名逻辑设备)")]
        public string HardDeviceID
        {
            get { return mHardDeviceID; }
            set { mHardDeviceID = value; }
        }

        private string mHardPNPDeviceID;

        [Browsable(false)]
        [DisplayName("PNPDeviceId(指示逻辑设备的Win32即插即用设备标识符)")]
        public string HardPNPDeviceID
        {
            get { return mHardDeviceID; }
            set { mHardDeviceID = value; }
        }
    }
    internal class HardwareScan
    {
        public enum HardwareType
        {
            KeyboardPlate,
            ChannelPlate,
            PCIE,
            TMC
        }
        private enum HardwareEnum
        {
            // 硬件
            Win32_Processor, // CPU 处理器
            Win32_PhysicalMemory, // 物理内存条
            Win32_Keyboard, // 键盘
            Win32_PointingDevice, // 点输入设备，包括鼠标。
            Win32_FloppyDrive, // 软盘驱动器
            Win32_DiskDrive, // 硬盘驱动器
            Win32_CDROMDrive, // 光盘驱动器
            Win32_BaseBoard, // 主板
            Win32_BIOS, // BIOS 芯片
            Win32_ParallelPort, // 并口
            Win32_SerialPort, // 串口
            Win32_SerialPortConfiguration, // 串口配置
            Win32_SoundDevice, // 多媒体设置，一般指声卡。
            Win32_SystemSlot, // 主板插槽 (ISA & PCI & AGP)
            Win32_USBController, // USB 控制器
            Win32_NetworkAdapter, // 网络适配器
            Win32_NetworkAdapterConfiguration, // 网络适配器设置
            Win32_Printer, // 打印机
            Win32_PrinterConfiguration, // 打印机设置
            Win32_PrintJob, // 打印机任务
            Win32_TCPIPPrinterPort, // 打印机端口
            Win32_POTSModem, // MODEM
            Win32_POTSModemToSerialPort, // MODEM 端口
            Win32_DesktopMonitor, // 显示器
            Win32_DisplayConfiguration, // 显卡
            Win32_DisplayControllerConfiguration, // 显卡设置
            Win32_VideoController, // 显卡细节。
            Win32_VideoSettings, // 显卡支持的显示模式。

            // 操作系统
            Win32_TimeZone, // 时区
            Win32_SystemDriver, // 驱动程序
            Win32_DiskPartition, // 磁盘分区
            Win32_LogicalDisk, // 逻辑磁盘
            Win32_LogicalDiskToPartition, // 逻辑磁盘所在分区及始末位置。
            Win32_LogicalMemoryConfiguration, // 逻辑内存配置
            Win32_PageFile, // 系统页文件信息
            Win32_PageFileSetting, // 页文件设置
            Win32_BootConfiguration, // 系统启动配置
            Win32_ComputerSystem, // 计算机信息简要
            Win32_OperatingSystem, // 操作系统信息
            Win32_StartupCommand, // 系统自动启动程序
            Win32_Service, // 系统安装的服务
            Win32_Group, // 系统管理组
            Win32_GroupUser, // 系统组帐号
            Win32_UserAccount, // 用户帐号
            Win32_Process, // 系统进程
            Win32_Thread, // 系统线程
            Win32_Share, // 共享
            Win32_NetworkClient, // 已安装的网络客户端
            Win32_NetworkProtocol, // 已安装的网络协议
            Win32_PnPEntity,//all device
        }

        /// <summary>
        /// 硬件信息
        /// </summary>
        

        private HardwareScan()
        {

        }

        /// <summary>
        /// 指定的设备是否在线（非通讯方式验证是否在线）
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static bool IsHardwareOnline(HardwareType type)
        {
            string[] names = MulGetHardwareInfo(HardwareEnum.Win32_PnPEntity, "Name");
            string target = "Unknown";

            switch (type)
            {
                case HardwareType.KeyboardPlate:
                    target = "USB-SERIAL CH340 (COM4)";
                    break;
                case HardwareType.ChannelPlate:
                    target = "USB-SERIAL CH340 (COM2)";
                    break;
                case HardwareType.PCIE:
                    target = "GHzScopeX Acquisition Device Driver";
                    break;
                case HardwareType.TMC:

                    break;
                default:
                    target = "Unknown";
                    break;
            }
            if (names.Contains(target))
            {
                Console.WriteLine($"设备在线\t{target}");
                return true;
            }
            else
                Console.WriteLine($"设备不在线\t{target}\t\t{type}");
            return false;
        }


        /// <summary>
        /// 获取指定设备的信息(设备管理器中的信息汇总)
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static HardwareInfo GetHDinfo(HardwareType type)
        {
            HardwareInfo info = new HardwareInfo();
            string name = "Unknown";
            switch (type)
            {
                case HardwareType.KeyboardPlate:
                    name = "USB-SERIAL CH340 (COM4)";
                    break;
                case HardwareType.ChannelPlate:
                    name = "USB-SERIAL CH340 (COM2)";
                    break;
                case HardwareType.PCIE:
                    name = "GHzScopeX Acquisition Device Driver";
                    break;
                case HardwareType.TMC:
                    break;
                default:
                    name = "Unknown";
                    break;
            }

            using (ManagementObjectSearcher searcher = new ManagementObjectSearcher("root\\cimv2", "select * from Win32_PnPEntity"))
            {
                var hardInfos = searcher.Get();
                foreach (ManagementObject hardInfo in hardInfos)
                {
                    string NAME = "";
                    if (hardInfo.Properties["Name"].Value != null)
                        NAME = hardInfo.Properties["Name"].Value.ToString();
                    else
                        continue;
                    string hardType = GetHardType(name);
                    if (NAME == name)
                    {
                        info.HardDescription = hardInfo.Properties["Description"].Value.ToString();
                        info.HardName = NAME;
                        info.HardDeviceID = hardInfo.Properties["DeviceId"].Value.ToString();
                        info.HardPNPDeviceID = hardInfo.Properties["PNPDeviceID"].Value.ToString();
                        info.HardClassGuid = hardInfo.Properties["ClassGuid"].Value.ToString();
                        info.HardManufacturer = hardInfo.Properties["Manufacturer"].Value.ToString();
                        info.HardType = hardType;
                        break;
                    }
                }
                searcher.Dispose();
            }
            return info;
        }

        /// <summary>
        /// 获取所有在线的示波器设备信息
        /// </summary>
        /// <returns></returns>
        public static IEnumerator<HardwareInfo> GetAllHDinfo()
        {
            List<HardwareInfo> allInfolist = new List<HardwareInfo>();
            List<string> names = new List<string>() { "USB-SERIAL CH340 (COM2)", "USB-SERIAL CH340 (COM4)", "GHzScopeX Acquisition Device Driver" };

            using (ManagementObjectSearcher searcher = new ManagementObjectSearcher("root\\cimv2", "select * from Win32_PnPEntity"))
            {
                var hardInfos = searcher.Get();
                foreach (ManagementObject hardInfo in hardInfos)
                {
                    string NAME = "";
                    if (hardInfo.Properties["Name"].Value != null)
                        NAME = hardInfo.Properties["Name"].Value.ToString();
                    else
                        continue;
                    string hardType = GetHardType(NAME);
                    if (names.Contains(NAME))
                    {
                        HardwareInfo info = new HardwareInfo();
                        info.HardDescription = hardInfo.Properties["Description"].Value.ToString();
                        info.HardName = NAME;
                        info.HardDeviceID = hardInfo.Properties["DeviceId"].Value.ToString();
                        info.HardPNPDeviceID = hardInfo.Properties["PNPDeviceID"].Value.ToString();
                        info.HardClassGuid = hardInfo.Properties["ClassGuid"].Value.ToString();
                        info.HardManufacturer = hardInfo.Properties["Manufacturer"].Value.ToString();
                        info.HardType = hardType;
                        Console.WriteLine(NAME + "\n");
                        allInfolist.Add(info);
                    }
                }
                searcher.Dispose();
            }
            return allInfolist.GetEnumerator();
        }

        /// <summary>
        /// 获取硬件类型
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        private static string GetHardType(string name)
        {
            string hardType = "Unknown";
            if (name == "USB-SERIAL CH340 (COM4)")
                hardType = "KeyboardPlate";
            else if (name == "USB-SERIAL CH340 (COM2)")
                hardType = "ChannelPlate";
            else if (name == "GHzScopeX Acquisition Device Driver")
                hardType = "PCIE";
            else if (name == "TMC")
                hardType = "TMC";
            else hardType = "Unknown";

            return hardType;
        }

        /// <summary>
        /// WMI获取硬件信息 获取所有硬件类型为hardType的propKey值
        /// </summary>
        /// <param name="hardType"> WMI提供的API类.</param>
        /// <param name="propKey"> 想要获取的属性值.</param>
        /// <returns></returns>
        private static string[] MulGetHardwareInfo(HardwareEnum hardType, string propKey)
        {
            List<string> strs = new List<string>();
            try
            {
                using (ManagementObjectSearcher searcher = new ManagementObjectSearcher("select * from " + hardType))
                {
                    var hardInfos = searcher.Get();
                    foreach (ManagementObject hardInfo in hardInfos)
                    {
                        if (hardInfo.Properties[propKey].Value != null)
                            strs.Add(hardInfo.Properties[propKey].Value.ToString());
                    }
                    searcher.Dispose();
                }
                return strs.ToArray();
            }
            catch
            {
                return strs.ToArray();
            }
        }
    }
}
